/******************************************************************************
 * Icinga 2                                                                   *
 * Copyright (C) 2012 Icinga Development Team (http://www.icinga.org/)        *
 *                                                                            *
 * This program is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU General Public License                *
 * as published by the Free Software Foundation; either version 2             *
 * of the License, or (at your option) any later version.                     *
 *                                                                            *
 * This program is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with this program; if not, write to the Free Software Foundation     *
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ******************************************************************************/

#ifndef CONFIGHIVE_H
#define CONFIGHIVE_H

namespace icinga
{

/**
 * A collection of all configuration objects that belong to an application.
 *
 * @ingroup base
 */
class I2_BASE_API ConfigHive : public Object
{
public:
	typedef shared_ptr<ConfigHive> Ptr;
	typedef weak_ptr<ConfigHive> WeakPtr;

	typedef map<string, ConfigCollection::Ptr>::iterator CollectionIterator;
	typedef map<string, ConfigCollection::Ptr>::const_iterator CollectionConstIterator;
	map<string, ConfigCollection::Ptr> Collections;

	void AddObject(const ConfigObject::Ptr& object);
	void RemoveObject(const ConfigObject::Ptr& object);
	ConfigObject::Ptr GetObject(const string& type,
	    const string& name = string());
	ConfigCollection::Ptr GetCollection(const string& type);

	void ForEachObject(const string& type,
	    function<int (const EventArgs&)> callback);

	Observable<EventArgs> OnObjectCommitted;
	Observable<EventArgs> OnObjectRemoved;
};

}

#endif /* CONFIGHIVE_H */
